﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Buttons : MonoBehaviour {

	public Replay replay;
	public Play play;
	public Wand left_wand, right_wand;

	int current_setting = 0, num_settings = 7;
	bool settings_shown = false;

	// Handle hand controller button presses.
	void Update () {
		Hand free_hand = play.free_hand;
		Hand paddle_hand = play.paddle_hand;

		// Show last hit paddle position if button 2 pressed.
		if (false && paddle_hand.wand.button2_pressed())
			replay.replay ();

		// Grip button holds ball in free hand
		// or if playing game and robot is serving, it initiates robot serve.
		if (free_hand.wand.grip_press () && !free_hand.holding_ball ())
			if (play.playing_game && !play.player_serves ())
				play.serve ();
			else
				play.hold_ball (free_hand);
		// Button 1 holds ball in paddle hand.
		if (paddle_hand.wand.button1_pressed () && !paddle_hand.holding_ball ())
			play.hold_ball (paddle_hand);

		// Start game if button 1 pressed in free hand.
		if (free_hand.wand.button1_pressed ())
			play.start_game ();
		
		// Robot serve if trigger pressed on either hand.
		if (paddle_hand.wand.trigger_press() || free_hand.wand.trigger_press()) {
			play.serve ();
			settings_shown = false;
		}

		/*
		 * Obsolete thumbstick text user interface for settings.
		Vector2 ts = paddle_hand.wand.direction_click();
		if (ts == Vector2.zero)
			ts = free_hand.wand.direction_click ();
		if (ts != Vector2.zero) {
			if (!settings_shown)
				show_settings ();
			else if (ts.x > 0)
				change_setting (1);
			else if (ts.x < 0)
				change_setting (-1);
			else if (ts.y > 0)
				next_setting (-1);
			else if (ts.y < 0)
				next_setting (1);
		}
		*/
	}

	void switch_paddle_hand() {
		Hand paddle_hand = play.paddle_hand, free_hand = play.free_hand;
		Wand fw = free_hand.wand, pw = paddle_hand.wand;
		paddle_hand.switch_hands (fw);
		free_hand.wand = pw;
	}

	void switch_paddle_grip() {
		Hand paddle_hand = play.paddle_hand;
		paddle_hand.grip = (paddle_hand.grip == Hand.Grip.ShakeHands ?
							Hand.Grip.PenHolder : Hand.Grip.ShakeHands);
		paddle_hand.set_paddle_grip ();
	}

	void show_settings() {
		string nl = System.Environment.NewLine;
		string text = "";
		for (int i = 0; i < num_settings; ++i)
			text += setting_line (i) + nl;
		play.billboard.text = text;
		settings_shown = true;
	}

	string setting_line(int setting) {
		Serve s = play.serves.current_serve ();
		Robot r = play.robot;
		string line = "";
		if (setting == 0)
			line = "Robot serve: " + s.name;
		else if (setting == 1)
			line = "Rally placement: " + r.patterns [r.pattern];
		else if (setting == 2)
			line = "Rally speed: " + r.speeds [r.speed];
		else if (setting == 3)
			line = "Rally spin: " + r.spins [r.spin];
		else if (setting == 6)
			line = "Robot movement: " + r.movement_speeds [r.movement_speed];
		else if (setting == 4) {
			string h = (play.paddle_hand.wand == right_wand ? "right" : "left");
			line = "Paddle hand: " + h;
		} else if (setting == 5) {
			string g = (play.paddle_hand.grip == Hand.Grip.ShakeHands ? "shake hand" : "pen holder");
			line = "Paddle grip: " + g;
		}
		if (setting == current_setting)
			line = "> " + line;
		return line;
	}

	void change_setting(int direction) {
		Robot r = play.robot;
		if (current_setting == 0) { // Serve
			Serves s = play.serves;
			if (direction > 0)
				s.next_serve ();
			else
				s.previous_serve ();
		} else if (current_setting == 1) // rally pattern
			r.pattern = (r.pattern + direction + r.patterns.Length) % r.patterns.Length;
		else if (current_setting == 2) // rally speed
			r.speed = (r.speed + direction + r.speeds.Length) % r.speeds.Length;
		else if (current_setting == 3) // rally spin
			r.spin = (r.spin + direction + r.spins.Length) % r.spins.Length;
		else if (current_setting == 6) { // robot motion
			r.movement_speed = (r.movement_speed + direction + r.movement_speeds.Length) % r.movement_speeds.Length;
			r.set_movement_speed ();
		} else if (current_setting == 4) // paddle hand
			switch_paddle_hand();
		else if (current_setting == 5) // paddle grip
			switch_paddle_grip();
	
		show_settings ();
	}

	void next_setting(int direction) {
		current_setting = (current_setting + direction + num_settings) % num_settings;
		show_settings ();
	}
}
