﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Hand : MonoBehaviour {

	public Wand wand;

	public Ball held_ball;
	Vector3 ball_hold_position = new Vector3(0,0,-.11f); // Palm of hand.
	float vmin_ball_release = 1f;  // Minimum ball release speed.

	public Paddle held_paddle;
	public enum Grip {ShakeHands, PenHolder};
	public Grip grip = Grip.ShakeHands;
	Vector3 paddle_grip_position;
	Quaternion paddle_grip_rotation;
	public bool freeze_paddle = false;  // Used for instant replay.
	float max_hand_accel = 0f;  // max acceleration, for debugging

	void Start () {
		if (wand != null && held_paddle != null)
			set_paddle_grip ();
	}

	// Update is called once per frame
	void Update () {
		move_held_ball ();
	}

	// Set paddle orientation in hand.
	public void switch_hands(Wand w) {
		if (w == wand)
			return;
		wand = w;

		set_paddle_grip ();
	}

	public void set_paddle_grip() {
		// In SteamVR the z-axis of the hand controller is parallel to the controller handle
		// and the hand palm direction is the x-axis.

		// Paddle offset y-axis is from base of handle to tip of blade,
		// x-axis is from left to right edge of blade facing forehand paddle side with handle down.
		// z-axis is perpendicular to blade in backhand direction (Unity uses left-handed coord system).
		if (grip == Grip.ShakeHands)
			paddle_grip_rotation = Quaternion.AngleAxis(90f, new Vector3(0,0,1)) * Quaternion.AngleAxis(90.0f, new Vector3(1,0,0));
		else if (grip == Grip.PenHolder)
			paddle_grip_rotation = Quaternion.AngleAxis(180f, new Vector3(0,0,1)) * Quaternion.AngleAxis(90.0f, new Vector3(1,0,0));
		Vector3 paddle_offset = new Vector3 (0.0f, 0.0f, 0.0f);
		paddle_grip_position = paddle_grip_rotation * paddle_offset;
	}

	// Update paddle position to match hand controller position.
	public void update_paddle_position(float delta_t) {
		if (held_paddle && !freeze_paddle)
			align_paddle_to_hand ();
	}

	void align_paddle_to_hand() {

		Vector3 hp, hv, hav, ha;
		Quaternion hr;
		wand.predict_hand_motion (out hp, out hr, out hv, out hav, out ha);

		// Remember maximum hand acceleration for reporting.
		max_hand_accel =  Mathf.Max (max_hand_accel, ha.magnitude);

		// Compute paddle motion from hand motion.
		Vector3 paddle_position, paddle_velocity, paddle_angular_velocity;
		Quaternion paddle_rotation;
		hand_to_paddle_motion (hp, hr, hv, hav,
			out paddle_position, out paddle_rotation,
			out paddle_velocity, out paddle_angular_velocity);

		// Set new paddle position
		held_paddle.move(paddle_position, paddle_rotation, paddle_velocity, paddle_angular_velocity);
	}

	// Determine paddle motion from hand motion.
	void hand_to_paddle_motion(Vector3 hp, Quaternion hr, Vector3 hv, Vector3 hav,
		out Vector3 paddle_position, out Quaternion paddle_rotation,
		out Vector3 paddle_velocity, out Vector3 paddle_angular_velocity) {
		// Get position and rotation of paddle.
		Vector3 paddle_offset = hr * paddle_grip_position;
		paddle_position = hp + paddle_offset;
		paddle_rotation = hr * paddle_grip_rotation;

		// Find paddle center velocity in world coordinates.
		paddle_velocity = hv + Vector3.Cross (hav, paddle_offset);
		paddle_angular_velocity = hav;
	}


	void move_held_ball() {
		if (held_ball == null)
			return;
		Vector3 p, v, a;
		Quaternion r;
		wand.object_motion (ball_hold_position, out p, out r, out v, out a);

		/* Adjust ball position in hand
		if (OVRInput.Get (OVRInput.RawButton.Y)) {
			hold_position += Quaternion.Inverse (r) * (ball.transform.position - p);
			Debug.Log ("Hold pos " + hold_position.ToString("F3"));
			return;
		}
		*/

		held_ball.set_motion (new BallState (0f, p, v, Vector3.zero));
		held_ball.transform.rotation = r;

		// Release ball if tossed up.
		if (Vector3.Dot (v, a) < 0 && v.magnitude >= vmin_ball_release && v.y > 0f) {
			held_ball.freeze = false;
			held_ball = null;
		}
	}

	public bool holding_ball () {
		return held_ball != null;
	}

	public void hold_ball(Ball b) {
		held_ball = b;
		held_ball.freeze = true;
		Vector3 p, v, a;
		Quaternion r;
		wand.object_motion (ball_hold_position, out p, out r, out v, out a);
		held_ball.set_motion(new BallState(0, p, v, Vector3.zero));
	}
}
