﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Serve {
	public string name;
	Toss toss;
	Vector3 vball;  // Ball velocity after leaving paddle
	float spin;		// Ball top spin after leaving paddle
	float swing_duration = 0.4f; // seconds
	float max_grazing_slope = 3f;

	public Serve(string name, Toss toss, Vector3 vb, float spin) {
		this.name = name;
		this.toss = toss;
		this.vball = vb;
		this.spin = spin;
	}

	public Serve() {
	}

	public virtual void serve_ball(Ball ball, Play play) {
		
		BallState hit_point = toss.toss_ball (ball, play.ball_tracking);

		Vector3 wdrop = Vector3.zero;
		Vector3 wb = (spin / ball.radius) * Vector3.Cross (vball, Vector3.up).normalized;
		Bouncer bn = play.robot.paddle.forehand_rubber ();
		Vector3 pn, pv;
		bn.wall_motion (hit_point.velocity, vball, wdrop, wb,
						ball.radius, ball.inertia_coef, max_grazing_slope,
						out pv, out pn);

		Vector3 php = hit_point.position - ball.radius * pn;
		Stroke s = new Stroke (hit_point.time, php, pn, pv, 0.5f*swing_duration, 0.5f*swing_duration);
		play.robot.set_stroke(s, ball.motion.time);
	}
}

public class Toss {
	Vector3 position, velocity;
	float hit_height;

	public Toss(Vector3 position, Vector3 velocity, float hit_height) {
		// In room coordinates.
		this.position = position;
		this.velocity = velocity;
		this.hit_height = hit_height;
	}

	public BallState toss_ball(Ball ball, BallTracking track) {
		Transform room = ball.transform.parent;
		Vector3 rp = room.TransformPoint (position);
		Vector3 rv = room.TransformVector(velocity);
		ball.set_motion(new BallState(0f, rp, rv, Vector3.zero));
		bool serve = true;
		BallTrack bt = track.predict_ball_flight (ball, serve);
		BallState bs = bt.drop_position (hit_height);
		return bs;
	}
}
