﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEngine.UI;

public class SettingsUI : MonoBehaviour
{
	public GameObject general_settings, robot_settings;
	public Button show_general_settings, done_general_settings;
	public Button show_robot_settings, done_robot_settings;

	public Play play;

	public Toggle shakehands, penholder;
	public Toggle righthanded, lefthanded;
	public Toggle white_ball, orange_striped_ball;
	public Toggle hit_arrows;
	public GameObject arrows;

	public Toggle move_slow, move_medium, move_fast, move_unlimited;
	public Toggle rally_slow, rally_medium, rally_fast;
	public Toggle spin_top, spin_back, spin_none, spin_backtop, spin_varied, spin_varied_back;
	public Toggle place_forehand, place_backhand, place_random_forehand, place_random, place_diagonal, place_down_line, place_elbow, place_no_return;
	public Toggle serve_topspin, serve_backspin, serve_long, serve_short, serve_to_forehand, serve_to_backhand, serve_line, serve_volley, serve_random;

    void Start()
    {
		// Show table top control panels when button on edge of table pressed.
		show_general_settings.onClick.AddListener(delegate { show_settings("general"); } );
		show_robot_settings.onClick.AddListener(delegate { show_settings("robot"); } );
		done_general_settings.onClick.AddListener(delegate {general_settings.SetActive(false); } );
		done_robot_settings.onClick.AddListener(delegate {robot_settings.SetActive(false); } );

		// Make toggle buttons change settings as soon as they are clicked.
		shakehands.onValueChanged.AddListener(delegate {if (shakehands.isOn) set_paddle_grip(Hand.Grip.ShakeHands);});
		penholder.onValueChanged.AddListener(delegate {if (penholder.isOn) set_paddle_grip(Hand.Grip.PenHolder);});

		righthanded.onValueChanged.AddListener(delegate {if (righthanded.isOn) set_paddle_hand(true);});
		lefthanded.onValueChanged.AddListener(delegate {if (lefthanded.isOn) set_paddle_hand(false);});

		white_ball.onValueChanged.AddListener(delegate {if (white_ball.isOn) play.balls.set_ball_coloring("white");});
		orange_striped_ball.onValueChanged.AddListener(delegate {if (orange_striped_ball.isOn) play.balls.set_ball_coloring("orange striped");});

		hit_arrows.onValueChanged.AddListener(delegate {arrows.SetActive(hit_arrows.isOn);});

		move_slow.onValueChanged.AddListener(delegate {if (move_slow.isOn) set_robot_speed("Slow");});
		move_medium.onValueChanged.AddListener(delegate {if (move_medium.isOn) set_robot_speed("Medium");});
		move_fast.onValueChanged.AddListener(delegate {if (move_fast.isOn) set_robot_speed("Fast");});
		move_unlimited.onValueChanged.AddListener(delegate {if (move_unlimited.isOn) set_robot_speed("Unlimited");});

		rally_slow.onValueChanged.AddListener(delegate {if (rally_slow.isOn) set_rally_speed("Slow");});
		rally_medium.onValueChanged.AddListener(delegate {if (rally_medium.isOn) set_rally_speed("Medium");});
		rally_fast.onValueChanged.AddListener(delegate {if (rally_fast.isOn) set_rally_speed("Fast");});
	
		spin_top.onValueChanged.AddListener(delegate {if (spin_top.isOn) set_rally_spin("Top spin");});
		spin_back.onValueChanged.AddListener(delegate {if (spin_back.isOn) set_rally_spin("Back spin");});
		spin_none.onValueChanged.AddListener(delegate {if (spin_none.isOn) set_rally_spin("No spin");});
		spin_backtop.onValueChanged.AddListener(delegate {if (spin_backtop.isOn) set_rally_spin("1 back spin, then top spin");});
		spin_varied.onValueChanged.AddListener(delegate {if (spin_varied.isOn) set_rally_spin("Varied");});
		spin_varied_back.onValueChanged.AddListener(delegate {if (spin_varied_back.isOn) set_rally_spin("Varied back spin");});
	
		place_forehand.onValueChanged.AddListener(delegate {if (place_forehand.isOn) set_rally_pattern("Forehand");});
		place_backhand.onValueChanged.AddListener(delegate {if (place_backhand.isOn) set_rally_pattern("Backhand");});
		place_random_forehand.onValueChanged.AddListener(delegate {if (place_random_forehand.isOn) set_rally_pattern("Random forehand");});
		place_random.onValueChanged.AddListener(delegate {if (place_random.isOn) set_rally_pattern("Random");});
		place_diagonal.onValueChanged.AddListener(delegate {if (place_diagonal.isOn) set_rally_pattern("Diagonal");});
		place_down_line.onValueChanged.AddListener(delegate {if (place_down_line.isOn) set_rally_pattern("Down line");});
		place_elbow.onValueChanged.AddListener(delegate {if (place_elbow.isOn) set_rally_pattern("Elbow");});
		place_no_return.onValueChanged.AddListener(delegate {if (place_no_return.isOn) set_rally_pattern("No return");});

		serve_topspin.onValueChanged.AddListener(set_serve);
		serve_backspin.onValueChanged.AddListener(set_serve);
		serve_long.onValueChanged.AddListener(set_serve);
		serve_short.onValueChanged.AddListener(set_serve);
		serve_to_forehand.onValueChanged.AddListener(set_serve);
		serve_to_backhand.onValueChanged.AddListener(set_serve);
		serve_line.onValueChanged.AddListener(set_serve);
		serve_volley.onValueChanged.AddListener(set_serve);
		serve_random.onValueChanged.AddListener(set_serve);
    }

	void show_settings(string panel_name)
	{
		if (panel_name == "general") {
			general_settings.SetActive(! general_settings.activeSelf);
			robot_settings.SetActive(false);
		} else if (panel_name == "robot") {
			robot_settings.SetActive(! robot_settings.activeSelf);
			general_settings.SetActive(false);
		}
	}

	void set_paddle_grip(Hand.Grip grip)
	{
		Hand paddle_hand = play.paddle_hand;
		paddle_hand.grip = grip;
		paddle_hand.set_paddle_grip ();
	}

	void set_paddle_hand(bool right)
	{
		Hand paddle_hand = play.paddle_hand, free_hand = play.free_hand;
		Wand pw = paddle_hand.wand;
		if (right == pw.right)
			return;
		paddle_hand.switch_hands (free_hand.wand);
		free_hand.wand = pw;
	}

	void set_robot_speed(string speed)
	{
		Robot r = play.robot;
		r.movement_speed = Array.IndexOf(r.movement_speeds, speed);
		r.set_movement_speed ();
	}

	void set_rally_speed(string speed)
	{
		Robot r = play.robot;
		r.speed = Array.IndexOf(r.speeds, speed);
	}

	void set_rally_spin(string spin)
	{
		Robot r = play.robot;
		r.spin = Array.IndexOf(r.spins, spin);
	}

	void set_rally_pattern(string pattern)
	{
		Robot r = play.robot;
		r.pattern = Array.IndexOf(r.patterns, pattern);
	}

	void set_serve(bool ignore)
	{
		string spin = (serve_topspin.isOn ? "topspin" : "backspin");
		string fb = (serve_to_forehand.isOn ? "Forehand" : "Backhand");
		string line = (serve_line.isOn ? " line" : "");
		string name = "";
		if (serve_random.isOn)
			name = "Random";
		else if (serve_volley.isOn)
			name = fb + " " + spin + " volley";
		else if (serve_long.isOn && serve_topspin.isOn)
			name = fb + " long topspin" + line;		
		else if (serve_short.isOn && serve_backspin.isOn)
			name = fb + " short backspin" + line;
		if (name.Length > 0) {
			Serves s = play.serves;
			s.set_serve(name);
		}
	}

}
